﻿using System;
using System.Drawing;

namespace IFretBoard {
    public class FretBoard {
      
        public int NumStrings = 6;          // êîë-âî ñòðóí
        public int NumFrets;                // êîë-âî ïîêàçûâàåìûõ ëàäîâ default 7
        public float FretBoardWidth;        // Øèðèíà
        public float offsX;          // ñìåùåíèå
        public float offsY;

        public int[] HiglitedStrings = new int[32] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        public int[] HiglitedFret = new int[32] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };


        //  Êîýôôèöèåíòû ðàçìåðîâ
        public float FretToHeight = 2f;            // Ñîîòíîøåíèå øèðèíû ëàäà ê âûñîòå ãðèôà
        public int[] FretDots = { 5, 7, 9, 12 };   // òî÷êè íà ëàäàõ
        public float DotRadiusPerc = 0.08f;        // % ðàäèóñ òî÷êè
        public float PitchWidthPerc = 0.02f;      // % øèðèíà êîëîê
        public float HighlightPerc = 0.18f;       // подсветка

        // Öâåòà
        public Color clFredBoard = Color.FromArgb(255, 120, 90, 20);     // Öâåò ãðèôà
        public Color clPitch = Color.FromArgb(192, 222, 222, 222);       // Öâåò êîëîê
        public Color clDots = Color.FromArgb(255, 255, 255, 255);        // Öâåò òî÷åê
        public Color clString = Color.FromArgb(255, 240, 230, 200);      // Öâåò ñòðóíû
        public Color clHighlight = Color.FromArgb(192, 255, 96, 96);      // Öâåò ïîäñâå÷åííîé ñòðóíû
        public Color clHighlightString = Color.FromArgb(255, 255, 0, 0);      // Öâåò ïîäñâå÷åííîé ñòðóíû

        public Color clShadow = Color.FromArgb(192, 20, 20, 20);         // Òåíü

        //  êèñòè
        public Brush brFredBoard;  // Initialize on each redraw
        public Brush brDots;
        public Brush brHighlight;
        public Pen pPitchPen;
        public Pen pStringPen;
        public Pen pHighlight;    

        // Constructor
        public FretBoard(int tWidth, int tNumFrets = 7) 
        {
            FretBoardWidth = tWidth;
            NumFrets = tNumFrets;
        }


        public void DrawToBmp(Graphics gBMP) 
        {

            gBMP.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            // Calculate sizes
            float FretBoardHeight = GetFretboardHeight();  // Âûñîòà ãðèôà
            float FretWidth = GetFretWidth();;                       // Øèðèíà ëàäà
             float PitchWidth = FretBoardHeight * PitchWidthPerc;                // Øèðèíà êîëîê

            // Initialize brush 
             brFredBoard = new SolidBrush(clFredBoard);  
             brDots = new SolidBrush(clDots);
             brHighlight = new SolidBrush(clHighlight);
             pPitchPen = new Pen(clPitch, PitchWidth);
             pStringPen = new Pen(clString, 2);
             pHighlight = new Pen(clHighlight, 5);
            // Do drawing

            // FretBoard background 
            gBMP.FillRectangle(brFredBoard, offsX, offsY, FretBoardWidth, FretBoardHeight); 
            
            // Pitch / Êîëêè
            for (int i = 0; i < NumFrets; i++)
            {
                pPitchPen.Color = clShadow;
                gBMP.DrawLine(pPitchPen, offsX + (i * FretWidth - 1),offsY + 1, offsX + (i * FretWidth - 1), offsY + ( FretBoardHeight - 1));
                pPitchPen.Color = clPitch;
                gBMP.DrawLine(pPitchPen, offsX + (i * FretWidth), offsY + 1, offsX + (i * FretWidth), offsY +  (FretBoardHeight - 1));
            };
            // Dots / áåëûå òî÷êè
            foreach (int i in FretDots)
            {
                if (i <= NumFrets)
                {
                    float dotXcenter = GetFretCenterX(i);
                    float dotYcenter = offsY + ((FretBoardHeight / 2));
                    float dotRadius = (FretWidth * DotRadiusPerc);
                    gBMP.FillEllipse(brDots, dotXcenter - dotRadius, dotYcenter - dotRadius, dotRadius * 2, dotRadius * 2);
                };
            };
            // Strings
            for (int i = 0; i < NumStrings; i++)
            {
                float y_half_offs = FretBoardHeight / (NumStrings * 2);
                pStringPen.Width = i / 1.5f;
                pStringPen.Color = clShadow;
                gBMP.DrawLine(pStringPen, offsX + 0, GetStringYPos(i)-2, offsX + FretBoardWidth, GetStringYPos(i)-2);

                if (HiglitedStrings[i] == 1) // ïîäñâå÷åííàÿ ñòðóíà
                {
                    pStringPen.Color = clHighlightString; //clHighlight;
                    pStringPen.Width = pStringPen.Width +1;
                }
                else {
                    pStringPen.Color = clString;
                };
                gBMP.DrawLine(pStringPen, offsX + 0, GetStringYPos(i), offsX + FretBoardWidth, GetStringYPos(i));

                 if(HiglitedFret[i] !=0 && HiglitedFret[i] <= NumFrets){
                     float HdotRadius = (FretWidth * HighlightPerc);
                      gBMP.FillEllipse(brHighlight, GetFretCenterX(HiglitedFret[i]) - HdotRadius, GetStringYPos(i) - HdotRadius, HdotRadius * 2, HdotRadius * 2);
                };
                
            };
               
        }
        // end of DrawToBmp
        float GetStringYPos(int gstring) { 
         float y_half_offs = GetFretboardHeight() / (NumStrings * 2);
            return offsY + ((gstring * y_half_offs * 2) + y_half_offs);
        }

        public float GetFretboardHeight(){
          return  (FretBoardWidth / NumFrets) * FretToHeight;;
        }

        public float GetFretCenterX(int fret) {
            return offsX + ((fret * GetFretWidth()) - (GetFretWidth() / 2));
        }

        public float GetFretWidth()
        {
            return FretBoardWidth / NumFrets;
        }

        public void ResetHighlight(){
            for (int i = 0; i < HiglitedStrings.Length; i++ )
            {
                HiglitedStrings[i] = 0;
                HiglitedFret[i] = 0;
            };

        }// end of reset

    }
}